# mappi/cursor

[![build status](https://gitlab.grifart.cz/jkuchar1/mappi-cursor/badges/master/build.svg)](https://gitlab.grifart.cz/jkuchar1/mappi-cursor/commits/master)

Provides API for reading data from database using cursors. This allows you to read huge data without all of them into memory.

Example usage:
````php
use Grifart\Mappi\Cursor;
$connection = new \Dibi\Connection([/* ... */]);
$cursorFactory = new Cursor\Driver\PostgresCursorFactory($connection);

// cursor allow to do basic operations
$cursor = $cursorFactory->create("SELECT * FROM mytable", TRUE);

// tracked cursor can tell current position
$trackedCursor = new Cursor\TrackedCursor($cursor, Cursor\Position::fromLeft(0));

// semantic cursor provides nicer API for cursor
$semanticCursor = new Cursor\SemanticCursor($cursor);

while($row = $semanticCursor->fetchNext()) {
    echo $trackedCursor->getPosition();
    print_r($row);
    echo "<hr>";
}

while($row = $semanticCursor->fetchPrevious()) {
    echo $trackedCursor->getPosition();
    print_r($row);
    echo "<hr>";
}
````