<?php declare(strict_types = 1);
/**
 * This file is part of mappi/cursor.
 */

namespace Grifart\Mappi\Cursor;

use Exception;

class CursorException extends \LogicException
{
	const MESSAGE_NO_DATA_TO_FETCH = "There was not data to fetch. Haven't you reached end of cursor?";
	const MESSAGE_OVERFLOW = "Cursor overflow. You've hit end or beginning of the cursor.";
	const MESSAGE_NO_DATA_CURSOR_RESET = "No data received. Cursor has been reset to zero.";
	const MESSAGE_CANNOT_RECOVER_TO_ORIGINAL_POSITION = "Cursor is in undefined state. " .
		"Error occurred and system tried to reset cursor into state before operation. " .
		"Unfortunately it failed. Original exception attached.";
	const MESSAGE_UNTRACEABLE_VALUE = "Given value is not traceable. %s";
	const MESSAGE_CANNOT_REACH_THE_END = "Cannot move cursor to end. Cursor was " .
        "in undirected state. Cursor has been left in unknown state.";

	/**
	 * @inheritDoc
	 */
	public function __construct($message, $code=0, \Exception $previous = null)
	{
		parent::__construct($message, $code, $previous);
	}

	public static function noDataToFetch() : self
	{
		return new static(self::MESSAGE_NO_DATA_TO_FETCH);
	}

	public static function cursorOverflow() : self
	{
		return new static(self::MESSAGE_OVERFLOW);
	}

	public static function noDataCursorHasBeenSetToZero() : self
	{
		return new static(self::MESSAGE_NO_DATA_CURSOR_RESET);
	}

	public static function cannotRecoverCursorIntoOriginalPosition(\Throwable $previous) : self
	{
		return new static(self::MESSAGE_CANNOT_RECOVER_TO_ORIGINAL_POSITION, 0, $previous);
	}

	public static function cannotMoveToTheEnd() : self
	{
		return new static(self::MESSAGE_CANNOT_REACH_THE_END);
	}

	public static function cursorPosition_invalidStartingPoint() : self
	{
		return new static("Invalid starting point value. Use constants defined in value object.");
	}

	public static function cursorPosition_invalidIndexValue(int $value) : self
	{
		return new static(
			sprintf("Index value %d is not valid.", $value)
		);
	}

	public static function untraceableValue($moreInfo) : self
	{
		return new static(
			sprintf(self::MESSAGE_UNTRACEABLE_VALUE, $moreInfo)
		);
	}

	public static function unexpectedRowCountReturned(int $rowCount) : self
	{
		return new static(
			sprintf("Underlying driver failed. Returned unexpected number (%d) of rows.", $rowCount)
		);
	}

	public static function cursorPosition_unknownError() : self
	{
		return new static("Moving cursor failed due to unknown error.");
	}

	public static function noResultsAvailable_hitBeginning() : self
	{
		return new static("No results available. You've hit the beginning.");
	}

	public static function noResultsAvailable_hitEnd() : self
	{
		return new static("No results available. You've hit the end.");
	}

}