<?php declare(strict_types = 1);
/**
 * This file is part of mappi/cursor.
 */

namespace Grifart\Mappi\Cursor;

/**
 * Value Object that holds position of cursor
 * use by {@see TrackedCursor}
 *
 * @link    https://github.com/nicolopignatelli/valueobjects (inspiration)
 * @package Grifart\Mappi\Cursor
 */
final class Position
{
	/** @var PositionOrigin */
	private $origin;

	/** @var int the cursor position */
	private $position = 0;

	/**
	 * @param PositionOrigin $origin   from left or from right?
	 * @param int            $position which position
	 */
	private function __construct(PositionOrigin $origin, int $position)
	{
		if($position < 0) {
			throw CursorException::cursorPosition_invalidIndexValue($position);
		}
		$this->origin = $origin;
		$this->position = $position;
	}

	public static function fromLeft(int $position) : self
	{
		return new self(
			PositionOrigin::get(PositionOrigin::FROM_LEFT),
			$position
		);
	}

	public static function fromRight(int $position) : self
	{
		return new self(
			PositionOrigin::get(PositionOrigin::FROM_RIGHT),
			$position
		);
	}
	
	public static function relativeTo(Position $position, int $offset) : self
	{
		$positionDelta = ($position->getOrigin()
			->is(PositionOrigin::FROM_LEFT) ? 1 : -1) * $offset;
		return new self(
			$position->getOrigin(),
			$position->getPosition() + $positionDelta
		);
	}

	/**
	 * From which side is position counted?
	 * @return PositionOrigin
	 */
	public function getOrigin()
	{
		return $this->origin;
	}

	/**
	 * Cursor position from origin
	 * @return int
	 */
	public function getPosition()
	{
		return $this->position;
	}

	function __toString() : string
	{
		return ($this->origin->getValue() === PositionOrigin::FROM_LEFT ? "left:" : "right:") . $this->position;
	}
}