<?php declare(strict_types = 1);
/**
 * @testCase
 */

// TODO: Maybe construct Cursor class in TrackedCursor constructor?

namespace Grifart\Mappi\Tests\Cursor;

use Grifart\Mappi\Cursor\Cursor;
use Grifart\Mappi\Cursor\ICursor;
use Grifart\Mappi\Cursor\Position;
use Grifart\Mappi\Cursor\TrackedCursor;
use Tester\Assert;

require_once __DIR__ . "/../bootstrap.php";
require_once __DIR__ . "/ICursorTest.php";

class TrackedCursorTest extends ICursorTest
{
	/** @var TrackedCursor */
	protected $uut;

	// test with underlying postgres driver
//	protected function setUp()
//	{
//		global $connection, $SQL_thousandRowsAscending;
//		try{
//			$connection->begin();
//
//			$factory = new PostgresCursorFactory($connection);
//			$cursor = $factory->create($SQL_thousandRowsAscending, true);
//			$this->uut = new TrackedCursor($cursor, Position::fromLeft(0));
//		} catch (DriverException $e) {
//			Environment::skip("It looks like you haven't properly set-up dibi connection to PostgreSQL. Check tests/bootstrap.php");
//		}
//
//		parent::setUp();
//	}
//
//	public function tearDown()
//	{
//		global $connection;
//		$connection->rollback();
//
//		parent::tearDown();
//	}

	protected function setUp()
	{

		$driver = new Driver\ArrayCursorDriver();
		$driver->createTestCursor("test", 1000);
		$cursor = new Cursor(
			$driver,
			"test"
		);

		$this->uut = new TrackedCursor(
			$cursor,
			Position::fromLeft(0)
		);
		parent::setUp();
	}

	private function assertPosition(string $position)
	{
		Assert::equal($position, (string)$this->uut->getPosition());
	}

	public function test_getPosition_left_initial()
	{
		$this->assertPosition("left:0");
	}

	public function test_getPosition_right_initial()
	{
		$this->uut->moveFromEndTo(0);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_moveTo()
	{
		$this->uut->moveTo(0);
		$this->assertPosition("left:0");
		$this->uut->moveTo(1);
		$this->assertPosition("left:1");
		$this->uut->moveTo(5);
		$this->assertPosition("left:5");
		$this->uut->moveTo(0);
		$this->assertPosition("left:0");

		$this->uut->moveTo(1000);
		$this->assertPosition("left:1000");

		$this->uut->moveTo(1001);
		$this->assertPosition("right:0");

		$this->uut->moveTo(9999);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_moveFromEndTo()
	{
		$this->uut->moveFromEndTo(0);
		$this->assertPosition("right:0");
		$this->uut->moveFromEndTo(1);
		$this->assertPosition("right:1");
		$this->uut->moveFromEndTo(5);
		$this->assertPosition("right:5");
		$this->uut->moveFromEndTo(0);
		$this->assertPosition("right:0");

		$this->uut->moveFromEndTo(1000);
		$this->assertPosition("right:1000");

		$this->uut->moveFromEndTo(1001);
		$this->assertPosition("left:0");

		$this->uut->moveFromEndTo(9999);
		$this->assertPosition("left:0");
	}

	public function test_getPosition_left_moveBy()
	{
		$this->uut->moveBy(-5);
		$this->assertPosition("left:0");

		$this->uut->moveBy(0);
		$this->assertPosition("left:0");

		$this->uut->moveBy(1);
		$this->assertPosition("left:1");

		$this->uut->moveBy(0);
		$this->assertPosition("left:1");

		$this->uut->moveBy(999);
		$this->assertPosition("left:1000");

		$this->uut->moveBy(1);
		$this->assertPosition("right:0");

		$this->uut->moveBy(1);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_right_moveBy()
	{
		$this->uut->moveFromEndTo(0);
		$this->assertPosition("right:0");

		$this->uut->moveBy(1);
		$this->assertPosition("right:0");

		$this->uut->moveBy(0);
		$this->assertPosition("right:0");

		$this->uut->moveBy(-5);
		$this->assertPosition("right:5");

		$this->uut->moveBy(-995);
		$this->assertPosition("right:1000");
		Assert::equal(1, $this->helper_fetchCurrentSingle()); // regression

		$this->uut->moveBy(-1);
		$this->assertPosition("left:0");
		Assert::null($this->helper_fetchCurrentSingle()); // regression

		$this->uut->moveBy(-1);
		$this->assertPosition("left:0");
	}

	public function test_getPosition_left_scroll()
	{
		Assert::same(0, $this->uut->scroll(-5));
		$this->assertPosition("left:0");

		Assert::same(0, $this->uut->scroll(0));
		$this->assertPosition("left:0");

		Assert::same(1, $this->uut->scroll(1));
		$this->assertPosition("left:1");

		Assert::same(0, $this->uut->scroll(0));
		$this->assertPosition("left:1");

		Assert::same(0, $this->uut->scroll(-1));
		$this->assertPosition("left:0");

		Assert::same(1, $this->uut->scroll(1));
		$this->assertPosition("left:1");
		
		Assert::same(999, $this->uut->scroll(999));
		Assert::same(1000, $this->helper_fetchCurrentSingle()); // regression
		$this->assertPosition("left:1000");

		Assert::same(0, $this->uut->scroll(0));
		$this->assertPosition("left:1000");
		
		Assert::same(0, $this->uut->scroll(1));
		$this->assertPosition("left:1001");

		Assert::same(0, $this->uut->scroll(1));
		$this->assertPosition("left:1001");

		Assert::same(1, $this->uut->scroll(-1));
		$this->assertPosition("left:1000");

		Assert::same(0, $this->uut->scroll(1));
		$this->assertPosition("left:1001");

		Assert::same(1000, $this->uut->scroll(ICursor::FETCH_FOREGOING));
		$this->assertPosition("left:0");

		Assert::same(1000, $this->uut->scroll(ICursor::FETCH_REMAINING));
		$this->assertPosition("left:1001");

		Assert::same(1000, $this->uut->scroll(ICursor::FETCH_FOREGOING));
		$this->assertPosition("left:0");
	}
	
	public function test_getPosition_right_scroll()
	{
		$this->uut->moveFromEndTo(0);
		$this->assertPosition("right:0");

		$this->uut->scroll(1);
		$this->assertPosition("right:0");

		$this->uut->scroll(0);
		$this->assertPosition("right:0");

		$this->uut->scroll(-5);
		$this->assertPosition("right:5");

		$this->uut->scroll(-995);
		$this->assertPosition("right:1000");
		Assert::equal(1, $this->helper_fetchCurrentSingle()); // regression

		$this->uut->scroll(-1);
		$this->assertPosition("right:1001");
		Assert::null($this->helper_fetchCurrentSingle()); // regression

		$this->uut->scroll(-1);
		$this->assertPosition("right:1001");
	}

	public function test_getPosition_left_fetchRange()
	{
		$this->assertPosition("left:0");
		$this->uut->fetchRange(0);
		$this->assertPosition("left:0");

		$this->uut->fetchRange(1);
		$this->assertPosition("left:1");
		$this->uut->fetchRange(0);
		$this->assertPosition("left:1");
		$this->uut->fetchRange(-1);
		$this->assertPosition("left:0");

		$this->uut->fetchRange(1000);
		$this->assertPosition("left:1000");

		$this->uut->fetchRange(1);
		$this->assertPosition("right:0");

		$this->uut->moveTo(999);
		$this->assertPosition("left:999");

		$this->uut->fetchRange(500);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_right_fetchRange()
	{
		$this->uut->moveFromEndTo(0);

		$this->assertPosition("right:0");
		$this->uut->fetchRange(0);
		$this->assertPosition("right:0");

		$this->uut->fetchRange(-1);
		$this->assertPosition("right:1");
		$this->uut->fetchRange(0);
		$this->assertPosition("right:1");
		$this->uut->fetchRange(+1);
		$this->assertPosition("right:0");

		$this->uut->fetchRange(-1000);
		$this->assertPosition("right:1000");

		$this->uut->fetchRange(-1);
		$this->assertPosition("left:0");

		$this->uut->moveFromEndTo(999);
		$this->assertPosition("right:999");

		$this->uut->fetchRange(-500);
		$this->assertPosition("left:0");
	}

	public function test_getPosition_left_fetchOneAt()
	{
		$this->assertPosition("left:0");
		$this->uut->fetchOneAt(0);
		$this->assertPosition("left:0");

		$this->uut->fetchOneAt(1);
		$this->assertPosition("left:1");

		$this->uut->fetchOneAt(500);
		$this->assertPosition("left:500");

		$this->uut->fetchOneAt(1000);
		$this->assertPosition("left:1000");

		$this->uut->fetchOneAt(1001);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_right_fetchOneAt()
	{
		$this->assertPosition("left:0");
		$this->uut->fetchOneAt(-1);
		$this->assertPosition("right:1");

		$this->uut->fetchOneAt(-1);
		$this->assertPosition("right:1");

		$this->uut->fetchOneAt(-500);
		$this->assertPosition("right:500");

		$this->uut->fetchOneAt(-1000);
		$this->assertPosition("right:1000");

		$this->uut->fetchOneAt(-1001);
		$this->assertPosition("left:0");
	}

	public function test_getPosition_left_fetchOneBy()
	{
		$this->uut->fetchOneBy(0);
		$this->assertPosition("left:0");

		$this->uut->fetchOneBy(-1);
		$this->assertPosition("left:0");

		$this->uut->fetchOneBy(1);
		$this->assertPosition("left:1");

		$this->uut->fetchOneBy(0);
		$this->assertPosition("left:1");

		$this->uut->fetchOneBy(-1);
		$this->assertPosition("left:0");

		$this->uut->fetchOneBy(20);
		$this->uut->fetchOneBy(480);
		$this->assertPosition("left:500");

		$this->uut->fetchOneBy(500);
		$this->assertPosition("left:1000");

		$this->uut->fetchOneBy(1);
		$this->assertPosition("right:0");
	}

	public function test_getPosition_right_fetchOneBy()
	{
		$this->uut->moveFromEndTo(0);

		$this->uut->fetchOneBy(0);
		$this->assertPosition("right:0");

		$this->uut->fetchOneBy(1);
		$this->assertPosition("right:0");

		$this->uut->fetchOneBy(-1);
		$this->assertPosition("right:1");

		$this->uut->fetchOneBy(0);
		$this->assertPosition("right:1");

		$this->uut->fetchOneBy(1);
		$this->assertPosition("right:0");

		$this->uut->fetchOneBy(-20);
		$this->uut->fetchOneBy(-480);
		$this->assertPosition("right:500");

		$this->uut->fetchOneBy(-500);
		$this->assertPosition("right:1000");

		$this->uut->fetchOneBy(-1);
		$this->assertPosition("left:0");
	}



	public function test_getPosition_givenInitialPosition_whenMove8Right_thenWillGet8Right()
	{
		$this->uut->moveBy(8);
		$this->assertPosition("left:8");
	}

	public function test_getPosition_givenInitialPosition_whenMove1Right_thenWillGet1Right()
	{
		$this->uut->moveBy(0);

		$this->uut->moveBy(1);
		$this->uut->moveBy(0);

		$this->assertPosition("left:1");
	}

	public function test_getPosition_givenInitialPosition_whenFetch3_thenWillBe3Right()
	{
		$this->uut->fetchRange(1);
		$this->uut->fetchRange(2);

		$this->assertPosition("left:3");
	}

	public function test_getPosition_givenRightEnd_whenFetch3Back_thenWillBe3Back()
	{
		$this->uut->moveFromEndTo(0);

		$this->uut->fetchRange(-1);
		$this->uut->fetchRange(-2);

		$this->assertPosition("right:3");
	}

	public function test_getPosition_givenRightEnd_whenGetPosition_thenOnRightEnd()
	{
		$this->uut->moveFromEndTo(0);

		$this->assertPosition("right:0");
	}

	public function test_getPosition_givenInMiddleFromRight_whenReadToRight_thenShou()
	{
		$this->uut->moveFromEndTo(0);

		$this->assertPosition("right:0");
	}

}

(new TrackedCursorTest())->run();